/*
 * This file is part of KQuickCharts
 * SPDX-FileCopyrightText: 2019 Arjen Hiemstra <ahiemstra@heimr.nl>
 * SPDX-FileCopyrightText: 2022 Alexander Lohnau <alexander.lohnau@gmx.de>
 *
 * SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */

#include <QtQuickTest>

#ifdef IMPORT_KQUICKCHARTS_QML_PLUGIN
#include <QPluginLoader>
Q_IMPORT_PLUGIN(QuickChartsPlugin)
#endif

QUICK_TEST_MAIN(Charts)
